@@ -1,4 +1,4 @@
-// Copyright 2024 The Hugo Authors. All rights reserved.
+// Copyright 2025 The Hugo Authors. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@ import (
 	"github.com/gohugoio/hugo/markup/asciidocext"
 	"github.com/gohugoio/hugo/markup/pandoc"
 	"github.com/gohugoio/hugo/markup/rst"
+	"github.com/gohugoio/hugo/related"
 )
 
 const filesPagesFromDataTempleBasic = `
@@ -73,10 +74,11 @@ Pfile Content
 {{ $title := printf "%s:%s" $pd $pp }}
 {{ $date := "2023-03-01" | time.AsTime }}
 {{ $dates := dict "date" $date }}
+{{ $keywords := slice "foo" "Bar"}}
 {{ $contentMarkdown := dict "value" "**Hello World**"  "mediaType" "text/markdown" }}
 {{ $contentMarkdownDefault := dict "value" "**Hello World Default**" }}
 {{ $contentHTML := dict "value" "<b>Hello World!</b> No **markdown** here." "mediaType" "text/html" }}
-{{ $.AddPage  (dict "kind" "page" "path" "P1" "title" $title "dates" $dates "content" $contentMarkdown "params" (dict "param1" "param1v" ) ) }}
+{{ $.AddPage  (dict "kind" "page" "path" "P1" "title" $title "dates" $dates "keywords" $keywords "content" $contentMarkdown "params" (dict "param1" "param1v" ) ) }}
 {{ $.AddPage  (dict "kind" "page" "path" "p2" "title" "p2title" "dates" $dates "content" $contentHTML ) }}
 {{ $.AddPage  (dict "kind" "page" "path" "p3" "title" "p3title" "dates" $dates "content" $contentMarkdownDefault "draft" false ) }}
 {{ $.AddPage  (dict "kind" "page" "path" "p4" "title" "p4title" "dates" $dates "content" $contentMarkdownDefault "draft" $data.draft ) }}
@@ -96,7 +98,8 @@ ADD_MORE_PLACEHOLDER
 
 func TestPagesFromGoTmplMisc(t *testing.T) {
 	t.Parallel()
-	b := hugolib.Test(t, filesPagesFromDataTempleBasic)
+	b := hugolib.Test(t, filesPagesFromDataTempleBasic, hugolib.TestOptWarn())
+	b.AssertLogContains("! WARN")
 	b.AssertPublishDir(`
 docs/p1/mytext.txt
 docs/p1/sub/mytex2.tx
@@ -329,6 +332,24 @@ func TestPagesFromGoTmplRemoveGoTmpl(t *testing.T) {
 	b.AssertFileContent("public/docs/index.html", "RegularPagesRecursive: pfile:/docs/pfile|$")
 }
 
+// Issue #13443.
+func TestPagesFromGoRelatedKeywords(t *testing.T) {
+	t.Parallel()
+	b := hugolib.Test(t, filesPagesFromDataTempleBasic)
+
+	p1 := b.H.Sites[0].RegularPages()[0]
+	icfg := related.IndexConfig{
+		Name: "keywords",
+	}
+	k, err := p1.RelatedKeywords(icfg)
+	b.Assert(err, qt.IsNil)
+	b.Assert(k, qt.DeepEquals, icfg.StringsToKeywords("foo", "Bar"))
+	icfg.Name = "title"
+	k, err = p1.RelatedKeywords(icfg)
+	b.Assert(err, qt.IsNil)
+	b.Assert(k, qt.DeepEquals, icfg.StringsToKeywords("p1:p1"))
+}
+
 func TestPagesFromGoTmplLanguagePerFile(t *testing.T) {
 	filesTemplate := `
 -- hugo.toml --
@@ -362,6 +383,28 @@ Single: {{ .Title }}|{{ .Content }}|
 	}
 }
 
+func TestPagesFromGoTmplDefaultPageSort(t *testing.T) {
+	t.Parallel()
+	files := `
+-- hugo.toml --
+defaultContentLanguage = "en"
+-- layouts/index.html --
+{{ range site.RegularPages }}{{ .RelPermalink }}|{{ end}}
+-- content/_content.gotmpl --
+{{ $.AddPage  (dict "kind" "page" "path" "docs/_p22" "title" "A" ) }}
+{{ $.AddPage  (dict "kind" "page" "path" "docs/p12" "title" "A" ) }}
+{{ $.AddPage  (dict "kind" "page" "path" "docs/_p12" "title" "A" ) }}
+-- content/docs/_content.gotmpl --
+{{ $.AddPage  (dict "kind" "page" "path" "_p21" "title" "A" ) }}
+{{ $.AddPage  (dict "kind" "page" "path" "p11" "title" "A" ) }}
+{{ $.AddPage  (dict "kind" "page" "path" "_p11" "title" "A" ) }}
+`
+
+	b := hugolib.Test(t, files)
+
+	b.AssertFileContent("public/index.html", "/docs/_p11/|/docs/_p12/|/docs/_p21/|/docs/_p22/|/docs/p11/|/docs/p12/|")
+}
+
 func TestPagesFromGoTmplEnableAllLanguages(t *testing.T) {
 	t.Parallel()
 
@@ -736,3 +779,45 @@ Single.
 
 	b.AssertFileContent("public/tags/index.html", "Terms: mytag: 1|§s")
 }
+
+func TestContentAdapterOutputsIssue13689(t *testing.T) {
+	t.Parallel()
+
+	files := `
+-- hugo.toml --
+disableKinds = ['home','rss','section','sitemap','taxonomy','term']
+[outputs]
+page = ['html','json']
+-- layouts/page.html --
+html: {{ .Title }}
+-- layouts/page.json --
+json: {{ .Title }}
+-- content/p1.md --
+---
+title: p1
+---
+-- content/p2.md --
+---
+title: p2
+outputs:
+  - html
+---
+-- content/_content.gotmpl --
+{{ $page := dict "path" "p3" "title" "p3" }}
+{{ $.AddPage $page }}
+
+{{ $page := dict "path" "p4" "title" "p4" "outputs" (slice "html") }}
+{{ $.AddPage $page }}
+`
+
+	b := hugolib.Test(t, files)
+
+	b.AssertFileExists("public/p1/index.html", true)
+	b.AssertFileExists("public/p1/index.json", true)
+	b.AssertFileExists("public/p2/index.html", true)
+	b.AssertFileExists("public/p2/index.json", false)
+	b.AssertFileExists("public/p3/index.html", true)
+	b.AssertFileExists("public/p3/index.json", true)
+	b.AssertFileExists("public/p4/index.html", true)
+	b.AssertFileExists("public/p4/index.json", false) // currently returns true
+}
