@@ -1,3 +1,18 @@
+// Copyright 2025 The Hugo Authors. All rights reserved.
+//
+// Portions Copyright The Go Authors.
+
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
 package tplimpl
 
 import (
@@ -107,6 +122,7 @@ func NewStore(opts StoreOptions, siteOpts SiteOptions) (*TemplateStore, error) {
 		treeMain:            doctree.NewSimpleTree[map[nodeKey]*TemplInfo](),
 		treeShortcodes:      doctree.NewSimpleTree[map[string]map[TemplateDescriptor]*TemplInfo](),
 		templatesByPath:     maps.NewCache[string, *TemplInfo](),
+		shortcodesByName:    maps.NewCache[string, *TemplInfo](),
 		cacheLookupPartials: maps.NewCache[string, *TemplInfo](),
 
 		// Note that the funcs passed below is just for name validation.
@@ -301,20 +317,23 @@ func (ti *TemplInfo) String() string {
 	return ti.PathInfo.String()
 }
 
-func (ti *TemplInfo) findBestMatchBaseof(s *TemplateStore, k1 string, slashCountK1 int, best *bestMatch) {
+func (ti *TemplInfo) findBestMatchBaseof(s *TemplateStore, d1 TemplateDescriptor, k1 string, slashCountK1 int, best *bestMatch) {
 	if ti.baseVariants == nil {
 		return
 	}
 
 	ti.baseVariants.WalkPath(k1, func(k2 string, v map[TemplateDescriptor]*TemplWithBaseApplied) (bool, error) {
+		if !s.inPath(k1, k2) {
+			return false, nil
+		}
 		slashCountK2 := strings.Count(k2, "/")
 		distance := slashCountK1 - slashCountK2
 
-		for d, vv := range v {
-			weight := s.dh.compareDescriptors(CategoryBaseof, ti.D, d)
+		for d2, vv := range v {
+			weight := s.dh.compareDescriptors(CategoryBaseof, false, d1, d2)
 			weight.distance = distance
 			if best.isBetter(weight, vv.Template) {
-				best.updateValues(weight, k2, d, vv.Template)
+				best.updateValues(weight, k2, d2, vv.Template)
 			}
 		}
 		return false, nil
@@ -378,11 +397,11 @@ func (q *TemplateQuery) init() {
 	} else if kinds.GetKindMain(q.Desc.Kind) == "" {
 		q.Desc.Kind = ""
 	}
-	if q.Desc.Layout == "" && q.Desc.Kind != "" {
+	if q.Desc.LayoutFromTemplate == "" && q.Desc.Kind != "" {
 		if q.Desc.Kind == kinds.KindPage {
-			q.Desc.Layout = layoutSingle
+			q.Desc.LayoutFromTemplate = layoutSingle
 		} else {
-			q.Desc.Layout = layoutList
+			q.Desc.LayoutFromTemplate = layoutList
 		}
 	}
 
@@ -404,9 +423,10 @@ type TemplateStore struct {
 	siteOpts   SiteOptions
 	htmlFormat output.Format
 
-	treeMain        *doctree.SimpleTree[map[nodeKey]*TemplInfo]
-	treeShortcodes  *doctree.SimpleTree[map[string]map[TemplateDescriptor]*TemplInfo]
-	templatesByPath *maps.Cache[string, *TemplInfo]
+	treeMain         *doctree.SimpleTree[map[nodeKey]*TemplInfo]
+	treeShortcodes   *doctree.SimpleTree[map[string]map[TemplateDescriptor]*TemplInfo]
+	templatesByPath  *maps.Cache[string, *TemplInfo]
+	shortcodesByName *maps.Cache[string, *TemplInfo]
 
 	dh descriptorHandler
 
@@ -447,7 +467,7 @@ func (s *TemplateStore) FindAllBaseTemplateCandidates(overlayKey string, desc Te
 				continue
 			}
 
-			if vv.D.isKindInLayout(desc.Layout) && s.dh.compareDescriptors(CategoryBaseof, descBaseof, vv.D).w1 > 0 {
+			if vv.D.isKindInLayout(desc.LayoutFromTemplate) && s.dh.compareDescriptors(CategoryBaseof, false, descBaseof, vv.D).w1 > 0 {
 				result = append(result, keyTemplateInfo{Key: k, Info: vv})
 			}
 		}
@@ -467,20 +487,31 @@ func (t *TemplateStore) ExecuteWithContext(ctx context.Context, ti *TemplInfo, w
 
 	templ := ti.Template
 
+	parent := tpl.Context.CurrentTemplate.Get(ctx)
+	var level int
+	if parent != nil {
+		level = parent.Level + 1
+	}
 	currentTi := &tpl.CurrentTemplateInfo{
-		Parent:                 tpl.Context.CurrentTemplate.Get(ctx),
+		Parent:                 parent,
+		Level:                  level,
 		CurrentTemplateInfoOps: ti,
 	}
 
 	ctx = tpl.Context.CurrentTemplate.Set(ctx, currentTi)
 
+	const levelThreshold = 999
+	if level > levelThreshold {
+		return fmt.Errorf("maximum template call stack size exceeded in %q", ti.Filename())
+	}
+
 	if t.opts.Metrics != nil {
 		defer t.opts.Metrics.MeasureSince(templ.Name(), time.Now())
 	}
 
 	execErr := t.storeSite.executer.ExecuteWithContext(ctx, ti, wr, data)
 	if execErr != nil {
-		return t.addFileContext(ti, execErr)
+		return t.addFileContext(ti, "execute of template failed", execErr)
 	}
 	return nil
 }
@@ -538,7 +569,7 @@ func (s *TemplateStore) LookupPagesLayout(q TemplateQuery) *TemplInfo {
 		return m
 	}
 	best1.reset()
-	m.findBestMatchBaseof(s, key, slashCountKey, best1)
+	m.findBestMatchBaseof(s, q.Desc, key, slashCountKey, best1)
 	if best1.w.w1 <= 0 {
 		return nil
 	}
@@ -547,20 +578,36 @@ func (s *TemplateStore) LookupPagesLayout(q TemplateQuery) *TemplInfo {
 
 func (s *TemplateStore) LookupPartial(pth string) *TemplInfo {
 	ti, _ := s.cacheLookupPartials.GetOrCreate(pth, func() (*TemplInfo, error) {
-		d := s.templateDescriptorFromPath(pth)
-		desc := d.Desc
-		if desc.Layout != "" {
-			panic("shortcode template descriptor must not have a layout")
+		pi := s.opts.PathParser.Parse(files.ComponentFolderLayouts, pth).ForType(paths.TypePartial)
+		k1, _, _, desc, err := s.toKeyCategoryAndDescriptor(pi)
+		if err != nil {
+			return nil, err
 		}
+		if desc.OutputFormat == "" && desc.MediaType == "" {
+			// Assume HTML.
+			desc.OutputFormat = s.htmlFormat.Name
+			desc.MediaType = s.htmlFormat.MediaType.Type
+			desc.IsPlainText = s.htmlFormat.IsPlainText
+		}
+
 		best := s.getBest()
 		defer s.putBest(best)
-		s.findBestMatchGet(s.key(path.Join(containerPartials, d.Path)), CategoryPartial, nil, desc, best)
+		s.findBestMatchGet(s.key(path.Join(containerPartials, k1)), CategoryPartial, nil, desc, best)
 		return best.templ, nil
 	})
 
 	return ti
 }
 
+func (s *TemplateStore) LookupShortcodeByName(name string) *TemplInfo {
+	name = strings.ToLower(name)
+	ti, _ := s.shortcodesByName.Get(name)
+	if ti == nil {
+		return nil
+	}
+	return ti
+}
+
 func (s *TemplateStore) LookupShortcode(q TemplateQuery) *TemplInfo {
 	q.init()
 	k1 := s.key(q.Path)
@@ -571,6 +618,9 @@ func (s *TemplateStore) LookupShortcode(q TemplateQuery) *TemplInfo {
 	defer s.putBest(best)
 
 	s.treeShortcodes.WalkPath(k1, func(k2 string, m map[string]map[TemplateDescriptor]*TemplInfo) (bool, error) {
+		if !s.inPath(k1, k2) {
+			return false, nil
+		}
 		slashCountK2 := strings.Count(k2, "/")
 		distance := slashCountK1 - slashCountK2
 
@@ -584,7 +634,7 @@ func (s *TemplateStore) LookupShortcode(q TemplateQuery) *TemplInfo {
 				continue
 			}
 
-			weight := s.dh.compareDescriptors(q.Category, q.Desc, k)
+			weight := s.dh.compareDescriptors(q.Category, vv.subCategory == SubCategoryEmbedded, q.Desc, k)
 			weight.distance = distance
 			if best.isBetter(weight, vv) {
 				best.updateValues(weight, k2, k, vv)
@@ -610,7 +660,7 @@ func (s *TemplateStore) PrintDebug(prefix string, category Category, w io.Writer
 			return
 		}
 		s := strings.ReplaceAll(strings.TrimSpace(vv.content), "\n", " ")
-		ts := fmt.Sprintf("kind: %q layout: %q content: %.30s", vv.D.Kind, vv.D.Layout, s)
+		ts := fmt.Sprintf("kind: %q layout: %q lang: %q content: %.30s", vv.D.Kind, vv.D.LayoutFromTemplate, vv.D.Lang, s)
 		fmt.Fprintf(w, "%s%s %s\n", strings.Repeat(" ", level), key, ts)
 	}
 	s.treeMain.WalkPrefix(prefix, func(key string, v map[nodeKey]*TemplInfo) (bool, error) {
@@ -662,6 +712,7 @@ func (s *TemplateStore) RefreshFiles(include func(fi hugofs.FileMetaInfo) bool)
 }
 
 func (s *TemplateStore) HasTemplate(templatePath string) bool {
+	templatePath = strings.ToLower(templatePath)
 	templatePath = paths.AddLeadingSlash(templatePath)
 	return s.templatesByPath.Contains(templatePath)
 }
@@ -690,20 +741,12 @@ func (t *TemplateStore) UnusedTemplates() []*TemplInfo {
 	var unused []*TemplInfo
 
 	for vv := range t.templates() {
-		if vv.subCategory != SubCategoryMain {
+		if vv.subCategory != SubCategoryMain || vv.isLegacyMapped {
 			// Skip inline partials and internal templates.
 			continue
 		}
-		if vv.noBaseOf {
-			if vv.executionCounter.Load() == 0 {
-				unused = append(unused, vv)
-			}
-		} else {
-			for vvv := range vv.BaseVariantsSeq() {
-				if vvv.Template.executionCounter.Load() == 0 {
-					unused = append(unused, vvv.Template)
-				}
-			}
+		if vv.executionCounter.Load() == 0 {
+			unused = append(unused, vv)
 		}
 	}
 
@@ -737,15 +780,25 @@ func (s *TemplateStore) findBestMatchGet(key string, category Category, consider
 			continue
 		}
 
-		weight := s.dh.compareDescriptors(category, desc, k.d)
+		weight := s.dh.compareDescriptors(category, vv.subCategory == SubCategoryEmbedded, desc, k.d)
 		if best.isBetter(weight, vv) {
 			best.updateValues(weight, key, k.d, vv)
 		}
 	}
 }
 
+func (s *TemplateStore) inPath(k1, k2 string) bool {
+	if k1 != k2 && !strings.HasPrefix(k1, k2+"/") {
+		return false
+	}
+	return true
+}
+
 func (s *TemplateStore) findBestMatchWalkPath(q TemplateQuery, k1 string, slashCountK1 int, best *bestMatch) {
 	s.treeMain.WalkPath(k1, func(k2 string, v map[nodeKey]*TemplInfo) (bool, error) {
+		if !s.inPath(k1, k2) {
+			return false, nil
+		}
 		slashCountK2 := strings.Count(k2, "/")
 		distance := slashCountK1 - slashCountK2
 
@@ -758,7 +811,7 @@ func (s *TemplateStore) findBestMatchWalkPath(q TemplateQuery, k1 string, slashC
 				continue
 			}
 
-			weight := s.dh.compareDescriptors(q.Category, q.Desc, k.d)
+			weight := s.dh.compareDescriptors(q.Category, vv.subCategory == SubCategoryEmbedded, q.Desc, k.d)
 
 			weight.distance = distance
 			isBetter := best.isBetter(weight, vv)
@@ -807,7 +860,7 @@ func (t *TemplateStore) addDeferredTemplate(owner *TemplInfo, name string, n *pa
 	return nil
 }
 
-func (s *TemplateStore) addFileContext(ti *TemplInfo, inerr error) error {
+func (s *TemplateStore) addFileContext(ti *TemplInfo, what string, inerr error) error {
 	if ti.Fi == nil {
 		return inerr
 	}
@@ -839,25 +892,27 @@ func (s *TemplateStore) addFileContext(ti *TemplInfo, inerr error) error {
 		fe := herrors.NewFileErrorFromName(inErr, fi.Meta().Filename)
 		fe.UpdateContent(f, lineMatcher)
 
-		if !fe.ErrorContext().Position.IsValid() {
-			return inErr, false
-		}
-		return fe, true
+		return fe, fe.ErrorContext().Position.IsValid()
 	}
 
-	inerr = fmt.Errorf("execute of template failed: %w", inerr)
+	inerr = fmt.Errorf("%s: %w", what, inerr)
 
-	if err, ok := checkFilename(ti.Fi, inerr); ok {
-		return err
+	var (
+		currentErr error
+		ok         bool
+	)
+
+	if currentErr, ok = checkFilename(ti.Fi, inerr); ok {
+		return currentErr
 	}
 
 	if ti.base != nil {
-		if err, ok := checkFilename(ti.base.Fi, inerr); ok {
-			return err
+		if currentErr, ok = checkFilename(ti.base.Fi, inerr); ok {
+			return currentErr
 		}
 	}
 
-	return inerr
+	return currentErr
 }
 
 func (s *TemplateStore) extractIdentifiers(line string) []string {
@@ -885,7 +940,7 @@ func (s *TemplateStore) extractInlinePartials() error {
 			name := templ.Name()
 			if !paths.HasExt(name) {
 				// Assume HTML. This in line with how the lookup works.
-				name = name + ".html"
+				name = name + s.htmlFormat.MediaType.FirstSuffix.FullSuffix
 			}
 			if !strings.HasPrefix(name, "_") {
 				name = "_" + name
@@ -1022,6 +1077,7 @@ func (s *TemplateStore) insertShortcode(pi *paths.Path, fi hugofs.FileMetaInfo,
 
 	m1[d] = ti
 
+	s.shortcodesByName.Set(k2, ti)
 	s.setTemplateByPath(pi.Path(), ti)
 
 	if fi != nil {
@@ -1063,6 +1119,12 @@ func (s *TemplateStore) insertTemplate2(
 		panic("category not set")
 	}
 
+	if category == CategoryPartial && d.OutputFormat == "" && d.MediaType == "" {
+		// See issue #13601.
+		d.OutputFormat = s.htmlFormat.Name
+		d.MediaType = s.htmlFormat.MediaType.Type
+	}
+
 	m := tree.Get(key)
 	nk := nodeKey{c: category, d: d}
 
@@ -1073,7 +1135,7 @@ func (s *TemplateStore) insertTemplate2(
 
 	if !replace {
 		if v, found := m[nk]; found {
-			if len(pi.IdentifiersUnknown()) >= len(v.PathInfo.IdentifiersUnknown()) {
+			if len(pi.Identifiers()) >= len(v.PathInfo.Identifiers()) {
 				// e.g. /pages/home.foo.html and  /pages/home.html where foo may be a valid language name in another site.
 				return nil, nil
 			}
@@ -1169,8 +1231,8 @@ func (s *TemplateStore) insertTemplates(include func(fi hugofs.FileMetaInfo) boo
 		case containerPartials, containerShortcodes, containerMarkup:
 			// OK.
 		default:
-			applyLegacyMapping = true
 			pi = fromLegacyPath(pi)
+			applyLegacyMapping = strings.Count(pi.Path(), "/") <= 2
 		}
 
 		if applyLegacyMapping {
@@ -1183,6 +1245,7 @@ func (s *TemplateStore) insertTemplates(include func(fi hugofs.FileMetaInfo) boo
 					ext:            pi.Ext(),
 					outputFormat:   pi.OutputFormat(),
 				}
+
 				if m2, ok := legacyOrdinalMappings[key]; ok {
 					if m1.ordinal < m2.m.ordinal {
 						// Higher up == better match.
@@ -1207,27 +1270,78 @@ func (s *TemplateStore) insertTemplates(include func(fi hugofs.FileMetaInfo) boo
 			)
 
 			base := piOrig.PathBeforeLangAndOutputFormatAndExt()
-			identifiers := pi.IdentifiersUnknown()
-
-			// Tokens on e.g. form /SECTIONKIND/THESECTION
-			insertSectionTokens := func(section string, kindOnly bool) string {
-				s := base
-				if !kindOnly {
-					s = strings.Replace(s, section, sectionToken, 1)
-				}
-				s = strings.Replace(s, kinds.KindSection, sectionKindToken, 1)
-				return s
+			identifiers := []string{}
+			if pi.Layout() != "" {
+				identifiers = append(identifiers, pi.Layout())
+			}
+			if pi.Kind() != "" {
+				identifiers = append(identifiers, pi.Kind())
 			}
 
-			for _, section := range identifiers {
-				if section == baseNameBaseof {
+			shouldIncludeSection := func(section string) bool {
+				switch section {
+				case containerShortcodes, containerPartials, containerMarkup:
+					return false
+				case "taxonomy", "":
+					return false
+				default:
+					for k, v := range s.opts.TaxonomySingularPlural {
+						if k == section || v == section {
+							return false
+						}
+					}
+					return true
+				}
+			}
+			if shouldIncludeSection(pi.Section()) {
+				identifiers = append(identifiers, pi.Section())
+			}
+
+			identifiers = helpers.UniqueStrings(identifiers)
+
+			// Tokens on e.g. form /SECTIONKIND/THESECTION
+			insertSectionTokens := func(section string) []string {
+				kindOnly := isLayoutStandard(section)
+				var ss []string
+				s1 := base
+				if !kindOnly {
+					s1 = strings.ReplaceAll(s1, section, sectionToken)
+				}
+				s1 = strings.ReplaceAll(s1, kinds.KindSection, sectionKindToken)
+				if s1 != base {
+					ss = append(ss, s1)
+				}
+				s1 = strings.ReplaceAll(base, kinds.KindSection, sectionKindToken)
+				if !kindOnly {
+					s1 = strings.ReplaceAll(s1, section, sectionToken)
+				}
+				if s1 != base {
+					ss = append(ss, s1)
+				}
+
+				helpers.UniqueStringsReuse(ss)
+
+				return ss
+			}
+
+			for _, id := range identifiers {
+				if id == "" {
 					continue
 				}
-				kindOnly := isLayoutStandard(section)
-				p := insertSectionTokens(section, kindOnly)
-				if m1, ok := s.opts.legacyMappingSection[p]; ok {
-					m1.mapping.targetPath = strings.Replace(m1.mapping.targetPath, sectionToken, section, 1)
-					handleMapping(m1)
+
+				p := insertSectionTokens(id)
+				for _, ss := range p {
+					if m1, ok := s.opts.legacyMappingSection[ss]; ok {
+						targetPath := m1.mapping.targetPath
+
+						if targetPath != "" {
+							targetPath = strings.ReplaceAll(targetPath, sectionToken, id)
+							targetPath = strings.ReplaceAll(targetPath, sectionKindToken, id)
+							targetPath = strings.ReplaceAll(targetPath, "//", "/")
+						}
+						m1.mapping.targetPath = targetPath
+						handleMapping(m1)
+					}
 				}
 			}
 
@@ -1325,7 +1439,7 @@ func (s *TemplateStore) parseTemplates() error {
 				if vv.state == processingStateTransformed {
 					continue
 				}
-				if err := s.tns.parseTemplate(vv); err != nil {
+				if err := s.parseTemplate(vv); err != nil {
 					return err
 				}
 			}
@@ -1345,7 +1459,7 @@ func (s *TemplateStore) parseTemplates() error {
 						// The regular expression used to detect if a template needs a base template has some
 						// rare false positives. Assume we don't need one.
 						vv.noBaseOf = true
-						if err := s.tns.parseTemplate(vv); err != nil {
+						if err := s.parseTemplate(vv); err != nil {
 							return err
 						}
 						continue
@@ -1374,7 +1488,7 @@ func (s *TemplateStore) parseTemplates() error {
 				if vvv.state == processingStateTransformed {
 					continue
 				}
-				if err := s.tns.parseTemplate(vvv); err != nil {
+				if err := s.parseTemplate(vvv); err != nil {
 					return err
 				}
 			}
@@ -1402,43 +1516,6 @@ type PathTemplateDescriptor struct {
 	Desc TemplateDescriptor
 }
 
-// templateDescriptorFromPath returns a template descriptor from the given path.
-// This is currently used in partial lookups only.
-func (s *TemplateStore) templateDescriptorFromPath(pth string) PathTemplateDescriptor {
-	var (
-		mt media.Type
-		of output.Format
-	)
-
-	// Common cases.
-	dotCount := strings.Count(pth, ".")
-	if dotCount <= 1 {
-		if dotCount == 0 {
-			// Asume HTML.
-			of, mt = s.resolveOutputFormatAndOrMediaType("html", "")
-		} else {
-			pth = strings.TrimPrefix(pth, "/")
-			ext := path.Ext(pth)
-			pth = strings.TrimSuffix(pth, ext)
-			ext = ext[1:]
-			of, mt = s.resolveOutputFormatAndOrMediaType("", ext)
-		}
-	} else {
-		path := s.opts.PathParser.Parse(files.ComponentFolderLayouts, pth)
-		pth = path.PathNoIdentifier()
-		of, mt = s.resolveOutputFormatAndOrMediaType(path.OutputFormat(), path.Ext())
-	}
-
-	return PathTemplateDescriptor{
-		Path: pth,
-		Desc: TemplateDescriptor{
-			OutputFormat: of.Name,
-			MediaType:    mt.Type,
-			IsPlainText:  of.IsPlainText,
-		},
-	}
-}
-
 // resolveOutputFormatAndOrMediaType resolves the output format and/or media type
 // based on the given output format suffix and media type suffix.
 // Either of the suffixes can be empty, and the function will try to find a match
@@ -1475,6 +1552,9 @@ func (s *TemplateStore) resolveOutputFormatAndOrMediaType(ofs, mns string) (outp
 	return outputFormat, mediaType
 }
 
+// templates iterates over all templates in the store.
+// Note that for templates with one or more base templates applied,
+// we will yield the variants, e.g. the templates that's actually in use.
 func (s *TemplateStore) templates() iter.Seq[*TemplInfo] {
 	return func(yield func(*TemplInfo) bool) {
 		for _, v := range s.treeMain.All() {
@@ -1511,25 +1591,13 @@ func (s *TemplateStore) toKeyCategoryAndDescriptor(p *paths.Path) (string, strin
 	outputFormat, mediaType := s.resolveOutputFormatAndOrMediaType(p.OutputFormat(), p.Ext())
 	nameNoIdentifier := p.NameNoIdentifier()
 
-	var layout string
-	unknownids := p.IdentifiersUnknown()
-	if p.Type() == paths.TypeShortcode {
-		if len(unknownids) > 1 {
-			// The name is the last identifier.
-			layout = unknownids[len(unknownids)-2]
-		}
-	} else if len(unknownids) > 0 {
-		// Pick the last, closest to the base name.
-		layout = unknownids[len(unknownids)-1]
-	}
-
 	d := TemplateDescriptor{
-		Lang:         p.Lang(),
-		OutputFormat: p.OutputFormat(),
-		MediaType:    mediaType.Type,
-		Kind:         p.Kind(),
-		Layout:       layout,
-		IsPlainText:  outputFormat.IsPlainText,
+		Lang:               p.Lang(),
+		OutputFormat:       p.OutputFormat(),
+		MediaType:          mediaType.Type,
+		Kind:               p.Kind(),
+		LayoutFromTemplate: p.Layout(),
+		IsPlainText:        outputFormat.IsPlainText,
 	}
 
 	d.normalizeFromFile()
@@ -1562,12 +1630,13 @@ func (s *TemplateStore) toKeyCategoryAndDescriptor(p *paths.Path) (string, strin
 	}
 
 	if category == CategoryPartial {
-		d.Layout = ""
+		d.LayoutFromTemplate = ""
 		k1 = p.PathNoIdentifier()
 	}
 
 	if category == CategoryShortcode {
 		k1 = p.PathNoIdentifier()
+
 		parts := strings.Split(k1, "/"+containerShortcodes+"/")
 		k1 = parts[0]
 		if len(parts) > 1 {
@@ -1577,15 +1646,15 @@ func (s *TemplateStore) toKeyCategoryAndDescriptor(p *paths.Path) (string, strin
 	}
 
 	// Legacy layout for home page.
-	if d.Layout == "index" {
+	if d.LayoutFromTemplate == "index" {
 		if d.Kind == "" {
 			d.Kind = kinds.KindHome
 		}
-		d.Layout = ""
+		d.LayoutFromTemplate = ""
 	}
 
-	if d.Layout == d.Kind {
-		d.Layout = ""
+	if d.LayoutFromTemplate == d.Kind {
+		d.LayoutFromTemplate = ""
 	}
 
 	k1 = strings.TrimPrefix(k1, "/_default")
@@ -1596,7 +1665,7 @@ func (s *TemplateStore) toKeyCategoryAndDescriptor(p *paths.Path) (string, strin
 	if category == CategoryMarkup {
 		// We store all template nodes for a given directory on the same level.
 		k1 = strings.TrimSuffix(k1, "/_markup")
-		parts := strings.Split(d.Layout, "-")
+		parts := strings.Split(d.LayoutFromTemplate, "-")
 		if len(parts) < 2 {
 			return "", "", 0, TemplateDescriptor{}, fmt.Errorf("unrecognized render hook template")
 		}
@@ -1605,7 +1674,7 @@ func (s *TemplateStore) toKeyCategoryAndDescriptor(p *paths.Path) (string, strin
 		if len(parts) > 2 {
 			d.Variant2 = parts[2]
 		}
-		d.Layout = "" // This allows using page layout as part of the key for lookups.
+		d.LayoutFromTemplate = "" // This allows using page layout as part of the key for lookups.
 	}
 
 	return k1, k2, category, d, nil
@@ -1640,30 +1709,14 @@ func (s *TemplateStore) transformTemplates() error {
 		if vv.category == CategoryBaseof {
 			continue
 		}
-		if !vv.noBaseOf {
-			for vvv := range vv.BaseVariantsSeq() {
-				tctx, err := applyTemplateTransformers(vvv.Template, lookup)
-				if err != nil {
-					return err
-				}
-
-				for name, node := range tctx.deferNodes {
-					if err := s.addDeferredTemplate(vvv.Overlay, name, node); err != nil {
-						return err
-					}
-				}
-			}
-		} else {
-			tctx, err := applyTemplateTransformers(vv, lookup)
-			if err != nil {
+		tctx, err := applyTemplateTransformers(vv, lookup)
+		if err != nil {
+			return err
+		}
+		for name, node := range tctx.deferNodes {
+			if err := s.addDeferredTemplate(vv, name, node); err != nil {
 				return err
 			}
-
-			for name, node := range tctx.deferNodes {
-				if err := s.addDeferredTemplate(vv, name, node); err != nil {
-					return err
-				}
-			}
 		}
 	}
 
@@ -1776,14 +1829,19 @@ func (best *bestMatch) isBetter(w weight, ti *TemplInfo) bool {
 		// Anything is better than nothing.
 		return true
 	}
+
 	if w.w1 <= 0 {
+
 		if best.w.w1 <= 0 {
 			return ti.PathInfo.Path() < best.templ.PathInfo.Path()
 		}
 		return false
 	}
 
-	if best.w.w1 > 0 {
+	// Note that for render hook templates, we need to make
+	// the embedded render hook template wih if they're a better match,
+	// e.g. render-codeblock-goat.html.
+	if best.templ.category != CategoryMarkup && best.w.w1 > 0 {
 		currentBestIsEmbedded := best.templ.subCategory == SubCategoryEmbedded
 		if currentBestIsEmbedded {
 			if ti.subCategory != SubCategoryEmbedded {
@@ -1816,11 +1874,7 @@ func (best *bestMatch) isBetter(w weight, ti *TemplInfo) bool {
 			return true
 		}
 
-		if ti.D.Layout != "" && best.desc.Layout != "" {
-			return ti.D.Layout != layoutAll
-		}
-
-		return w.distance < best.w.distance || ti.PathInfo.Path() < best.templ.PathInfo.Path()
+		return ti.PathInfo.Path() < best.templ.PathInfo.Path()
 	}
 
 	return true
@@ -1868,17 +1922,6 @@ type weight struct {
 	distance int
 }
 
-func (w weight) isEqualWeights(other weight) bool {
-	return w.w1 == other.w1 && w.w2 == other.w2 && w.w3 == other.w3
-}
-
-func isLayoutCustom(s string) bool {
-	if s == "" || isLayoutStandard(s) {
-		return false
-	}
-	return true
-}
-
 func isLayoutStandard(s string) bool {
 	switch s {
 	case layoutAll, layoutList, layoutSingle:
@@ -1888,6 +1931,10 @@ func isLayoutStandard(s string) bool {
 	}
 }
 
+func (w weight) isEqualWeights(other weight) bool {
+	return w.w1 == other.w1 && w.w2 == other.w2 && w.w3 == other.w3
+}
+
 func configureSiteStorage(opts SiteOptions, watching bool) *storeSite {
 	funcsv := make(map[string]reflect.Value)
 
